//GLHTFBumpHDS.pas cloned..cut from GLHeightData
{: Classes for height data access...BumpMapping<p>

   Cloned from and BumpMap.<p>

   In short: access to raw height data is performed by a THeightDataSource
   subclass, that must take care of performing all necessary data access,
   cacheing and manipulation to provide THeightData objects. A THeightData
   is basicly a square, power of two dimensionned raster heightfield, and
   holds the data a renderer needs.<p>

	<b>History : </b><font size=-1><ul>

	   <li>11/02/01 - EG-ILH - Cloned Creation
	</ul></font>
}
unit GLHTFBumpHDS;

interface

{$i GLScene.inc}
                              //added  GLGraphics,  GLTexture, GLHeightData
uses Classes, VectorGeometry, GLHeightData, GLGraphics,  GLTexture, GLCrossPlatform;

type
	//TGLHTFBumpHDS from TGLCustomHDS
	//
   {: An Height Data Source for custom use.<p>
      Provides event handlers for the various requests to be implemented
      application-side (for application-specific needs). }
	TGLHTFBumpHDS = class (TGLCustomHDS)
	   private
	      { Private Declarations }
         FOnMarkDirty : TMarkDirtyEvent;
         //Added
         FBumpmapLibrary : TGLMaterialLibrary;
         FBumpScale : Single;
         FSubSampling : Integer;
	   protected
	      { Protected Declarations }
         //procedure StartPreparingData(heightData : THeightData); override;
         //added
         procedure SetBumpmapLibrary(const val : TGLMaterialLibrary);
         procedure SetBumpScale(const val : Single);
         function StoreBumpScale : Boolean;
         procedure SetSubSampling(const val : Integer);

	   public
	      { Public Declarations }
	      constructor Create(AOwner: TComponent); override;
         destructor Destroy; override;
         procedure MarkDirty(const area : TGLRect); override;
         //added
         procedure Notification(AComponent: TComponent; Operation: TOperation); override;

	   published
	      { Published Declarations }
         //property MaxPoolSize;

        // property OnStartPreparingData : TStartPreparingDataEvent read FOnStartPreparingData write FOnStartPreparingData;
        property OnMarkDirtyEvent : TMarkDirtyEvent read FOnMarkDirty write FOnMarkDirty;
//added
         property BumpmapLibrary : TGLMaterialLibrary read FBumpmapLibrary write SetBumpmapLibrary;
         property BumpScale : Single read FBumpScale write SetBumpScale stored StoreBumpScale;
         {: Specifies the amount of subsampling for the bump texture.<p>
            This value must be a power of 2, and is used to divide the height
            tile resolution to determine the bump texture resolution (f.i.
            a tile size of 128 with a subsampling of 4 will result in textures
            of a resolution of 32x32. SubSampling won't allow texture resolution
            to get below 16x16 (minimal bumpmap resolution). }
         property SubSampling : Integer read FSubSampling write SetSubSampling default 1;

	end;

{procedure Register;}
// ------------------------------------------------------------------
// ------------------------------------------------------------------
// ------------------------------------------------------------------
implementation
// ------------------------------------------------------------------
// ------------------------------------------------------------------
// ------------------------------------------------------------------

uses SysUtils, GLMisc, ApplicationFileIO, GLUtils
  {$IFDEF MSWINDOWS}
  , Windows  // for CreateMonochromeBitmap
  {$ENDIF}
  ;

{procedure Register;
begin
  RegisterComponents('GLScene', [TGLHTFBumpHDS]);
end; }
// ------------------
// ------------------TGLHTFBumpHDS TGLCustomHDS ------------------
// ------------------
const
   cDefaultBumpScale = 0.1;//added
// Create
//
constructor TGLHTFBumpHDS.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
   FBumpScale:=cDefaultBumpScale;//added
   FSubSampling:=1;
end;

// Destroy
//
destructor TGLHTFBumpHDS.Destroy;
begin
   BumpmapLibrary:=nil;//added
	inherited Destroy;
end;

// MarkDirty
//
procedure TGLHTFBumpHDS.MarkDirty(const area : TGLRect);
begin
   inherited;
   if Assigned(FOnMarkDirty) then
      FOnMarkDirty(area);
end;

// Notification
//
procedure TGLHTFBumpHDS.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if Operation=opRemove then begin
       if AComponent=FBumpmapLibrary then
         BumpmapLibrary:=nil
    // else if AComponent=FElevationHDS then ElevationHDS:=nil
   end;
   inherited;
end;
// StartPreparingData
//
{procedure TGLHTFBumpHDS.StartPreparingData(heightData : THeightData);
begin
   if Assigned(FOnStartPreparingData) then
      FOnStartPreparingData(heightData);
   if heightData.DataState<>hdsNone then
      heightData.DataState:=hdsReady;
end;}

//added
// SetBumpmapLibrary
//
procedure TGLHTFBumpHDS.SetBumpmapLibrary(const val : TGLMaterialLibrary);
begin
   if val<>FBumpmapLibrary then begin
      if Assigned(FBumpmapLibrary) then
         FBumpmapLibrary.RemoveFreeNotification(Self);
      FBumpmapLibrary:=val;
      if Assigned(FBumpmapLibrary) then
         FBumpmapLibrary.FreeNotification(Self);
      MarkDirty;
   end;
end;

// SetBumpScale
//
procedure TGLHTFBumpHDS.SetBumpScale(const val : Single);
begin
   if FBumpScale<>val then begin
      FBumpScale:=val;
      MarkDirty;
   end;
end;

// StoreBumpScale
//
function TGLHTFBumpHDS.StoreBumpScale : Boolean;
begin
   Result:=(FBumpScale<>cDefaultBumpScale);
end;

// SetSubSampling
//
procedure TGLHTFBumpHDS.SetSubSampling(const val : Integer);
begin
   if val<>FSubSampling then begin
      FSubSampling:=RoundDownToPowerOf2(val);
      if FSubSampling<1 then
         FSubSampling:=1;
      MarkDirty;
   end;
end;



// ------------------------------------------------------------------
// ------------------------------------------------------------------
// ------------------------------------------------------------------
initialization
// ------------------------------------------------------------------
// ------------------------------------------------------------------
// ------------------------------------------------------------------

	// class registrations
 {  Classes.RegisterClasses([TGLHTFBumpHDS]); }

end.


