{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit GlsMathRandom;

interface

procedure BBrownian(Seed,Step: Longint);
procedure BBrown2d(Seed,Step: Longint);
procedure Brown_Gauss(F1, D: Extended; Time: Integer);
procedure WhiteBrown(Time: Integer);
procedure Cellar;
procedure CellTorus;
procedure CellZap;
procedure HiFiLines;
procedure Chaser(Seed,Step: Longint);

implementation

uses Math,GlsMathingsFrm;

procedure BBrownian(Seed,Step: Longint);
const
  scale: Extended = 2;//80;
  h: Extended = 0;//0.5;//0.87;
  mu: Extended = 0.0;
  sigma: Extended = 1.0;
  var
  Fh: array[0..101{256}] of Extended;
  ratio, std: Extended;
  //maxcolx, maxrowy, i: integer;
  function gauss(seed: longint; mu: Extended; sigma: Extended):
    Extended;
  var
    x: Extended;
    i: integer;
  begin
    if seed <> 0 then RandSeed := seed;
    x := 0;
    for i := 0 to 11 do x := x + Random;
    x := x - 6.0;
    gauss := mu + sigma * x;
  end;
  procedure subdivide(f1: integer; f2: integer; std: Extended);
  var
    fmid: integer;
    stdmid: Extended;
  begin
    fmid := (f1 + f2) div 2;
    if (fmid <> f1) and (fmid <> f2) then
    begin
      Fh[fmid] := (Fh[f1] + Fh[f2]) / 2.0 +
        gauss(0, mu, sigma) * std;
      stdmid := std * ratio;
      subdivide(f1, fmid, stdmid);
      subdivide(fmid, f2, stdmid);
    end;
  end;
Begin
    Fh[0] := gauss(seed, mu, sigma) * scale;
    Fh[101{256}] := gauss(0, mu, sigma) * scale;
    ratio := Exp(-0.693147 * h);
    std := scale * ratio;
    subdivide(0, 101{256}, std);
yr:=Fh[Step];
xr:= Fh[Step];
zr:= Fh[Step];
  AlphaRF:=AlphaRF-Incremental ;//(255);
  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);

End;

procedure BBrown2d(Seed,Step: Longint);
const
  scale: Extended = 2;//80;
  h: Extended = 0;//0.5;//0.87;
  mu: Extended = 0.0;
  sigma: Extended = 1.0;
  var
  Fh: array[0..101{256}] of Extended;
  Fw: array[0..101{256}] of Extended;
  ratio, std: Extended;
  //maxcolx, maxrowy, i: integer;
  function gauss(seed: longint): Extended;
  var
    x: Extended;
    i: integer;
  begin
    if seed <> 0 then RandSeed := seed;
    x := 0;
    for i := 0 to 11 do x := x + Random;
    x := x - 6.0;
    gauss := mu + sigma * x;
  end;
  procedure subdivide(f1: integer; f2: integer; std: Extended);
  var
    fmid: integer;
    stdmid: Extended;
  begin
    fmid := (f1 + f2) div 2;
    if (fmid <> f1) and (fmid <> f2) then begin
      Fh[fmid] := (Fh[f1] + Fh[f2]) / 2.0 + gauss(0) * std;
      Fw[fmid] := (Fw[f1] + Fw[f2]) / 2.0 + gauss(0) * std;
      stdmid := std * ratio;
      subdivide(f1, fmid, stdmid);
      subdivide(fmid, f2, stdmid);
    end;
  end;
Begin
    Fh[0] := gauss(seed) * scale;
    Fh[101] := gauss(0) * scale;
    Fw[0] := gauss(seed) * scale;
    Fw[101] := gauss(0) * scale;

    ratio := Exp(-0.693147 * h);
    std := scale * ratio;
    subdivide(0, 101{256}, std);
yr:=Fw[Step];
xr:= Fh[Step];
zr:= Fh[Step];
  AlphaRF:=AlphaRF-Incremental ;//(255);
  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);
End;

procedure Brown_Gauss(F1, D: Extended; Time: Integer);
var
  //maxcolx, maxrowy,
  N, I, X: Integer;
  F, PH: array[0..16] of Extended;
  RHO, SSQ, AC, DUM, RN, Z, Y: Extended;
Begin
  SSQ := 0.0;
  AC := 0.0;
  RN := 0.0;
  Z := 0.0;
    for N := 0 to 16 do
    begin
      F[N] := (EXP(N * ln(F1)));
      PH[N] := 6.28 * Random;
    end;
   X := Time;
    begin
      Y := 0.0;
      for N := 1 to 16 do
        Y := (Y + Time * SIN(F[N] * (X / Time) + PH[N])
          * (EXP((D - 2) * LN(F[N]))));

      SSQ := SSQ + Y * Y;
      AC := AC + Y * Z;
      Z := Y;
    end;
xr:= y;
//zr:= y ;
    RHO := AC / SSQ;
    DUM := SQRT((1 - RHO * RHO) * (SSQ / Time));
      for I := 0 to 11 do RN := RN + Random;
      RN := RN - 6;
      Y := Y * RHO + DUM * RN;
yr:=y;
      Y := Y * RHO + DUM * RN;
zr:= z*y;
//z+RHO;
  AlphaRF:=AlphaRF-Incremental ;//(255);
  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);
End;

procedure WhiteBrown(Time: Integer);
Begin
If (Time mod 2)=0 then
begin
yr:=(RandG(0.0, 1.0) * 2);
xr:=(RandG(0.0, 1.0) * 2);
zr:= (RandG(0.0, 1.0) * 2);
  AlphaRF:=AlphaRF-Incremental ;//(255);
  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);
end else
begin
yr:=(RandG(10.0, 2.0) * 2);
xr:= (RandG(10.0, 2.0) * 2);
zr:= (RandG(10.0, 2.0) * 2);
  AlphaRF:=AlphaRF-Incremental ;//(255);
  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);
end
End;

procedure Cellar;
Begin
  xr := xr + ((Random - 0.5) *wr);
  Yr := Yr + ((Random - 0.5) *wr);
  Zr := Zr + ((Random - 0.5) *wr);
        if Xr > maxx then Xr := maxx;
        if Xr < minx then Xr := minx;
        if Yr > maxy then Yr := maxy;
        if Yr < miny then Yr := miny;
        if Zr > maxZ then Zr := maxZ;
        if Zr < minZ then Zr := minZ;
  AlphaRF:=AlphaRF-Incremental ;//(255);
//  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);
End;

procedure CellTorus;
Begin
  xr := xr + ((Random - 0.5) *wr);
  Yr := Yr + ((Random - 0.5) *wr);
  Zr := Zr + ((Random - 0.5) *wr);
        if Xr > maxx then Xr := minx;
        if Xr < minx then Xr := maxx;
        if Yr > maxy then Yr := miny;
        if Yr < miny then Yr := maxy;
        if Zr > maxZ then Zr := minZ;
        if Zr < minZ then Zr := maxZ;
  AlphaRF:=AlphaRF-Incremental ;//(255);
//  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);
End;

procedure CellZap;
Begin
  xr := xr + ((Random - 0.5) *wr);
  Yr := Yr + ((Random - 0.5) *wr);
  Zr := Zr + ((Random - 0.5) *wr);
        if Xr > maxx then Xr := 0;
        if Xr < minx then Xr := 0;
        if Yr > maxy then Yr := 0;
        if Yr < miny then Yr := 0;
        if Zr > maxZ then Zr := 0;
        if Zr < minZ then Zr := 0;
  AlphaRF:=AlphaRF-Incremental ;//(255);
//  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);
End;

procedure HiFiLines;
Begin
  xr :=  (5 - (Random(11)));
  //xr + ((Random - 0.5) *wr);
  Yr :=(5 - (Random(11))) ;
   //Yr + ((Random - 0.5) *wr);
  Zr := (5 - (Random(11)));
  //Zr + ((Random - 0.5) *wr);
        if Xr > maxx then Xr := 0;
        if Xr < minx then Xr := 0;
        if Yr > maxy then Yr := 0;
        if Yr < miny then Yr := 0;
        if Zr > maxZ then Zr := 0;
        if Zr < minZ then Zr := 0;
  AlphaRF:=AlphaRF-Incremental ;//(255);
//  AlphaGF:=AlphaGF+Incremental ;//(255);
  AlphaBF:=AlphaBF+Incremental ;//(255);
End;

procedure Chaser(Seed,Step: Longint);
Begin
{}
End;





end.
