unit TreeModelParameters;

interface

uses Graphics;

type
     TModelParameters = class
     private
       FShape: byte;
       FLeveled: byte;
       FSeason: byte;
       FAttractionUp: Double;
       FTreeSeed : integer;
       FHealth: byte;           {determines percent Leaves}
       FDeathToll: byte;        {Where they die}

       FDistance : Double;      {Link to ~ what Leaf Quality was?}
       FWindStrength : byte;     {0..30 ?}
       FWindGust : byte;        {0..?}
       FWindFrom : integer;     {0..360}
       FSunHeight  : byte;      {0..90}
       FSunStrength : byte;    {0..100}
       FSunFrom  : integer;    {0..360}

       FTreeImageFile:String;
       FTreeShapeFile:String;
       FTreeSpriteFile:String;

       FScale0, FScaleV0: Double;
       FnLength, FnLengthV: array of Double;
       FRatio, FRatioPower: Double;
       FScale, FScaleV: Double;

       FLobes: byte;
       FLobeDepth: Double;
       FLobeScale, FLobeScaleV: Double;

       FTreeBarkFile:String;
       {Dunno how to set..texture overides.. use Ambient instead?}
       FTreeColor    : TColor;
       FTreeColorVariation : boolean;

       FBaseSplits: byte;
       FBaseSize: Double;
       FZScale, FZScaleV: Double; {Used for b splits}

       FBoleSplits: byte; {added}
       FBoleSize: Double;
       FFlare, FFlareV: Double;

       FnVertices: array of integer;  {Number of parts in Trunk radius}
       FnBranches : array of integer;
       {allows leaves on Every level, besides the last}
       FnLeaves : array of integer;
       FnTaper: array of Double;
       FnDownAngle, FnDownAngleV: array of Double;
       FnRotate, FnRotateV: array of Double;
       FnCurveRes: array of integer;
       FnCurve, FnCurveBack, FnCurveV: array of Double;
       FnSegSplits: array of Double;
       FnSplitAngle, FnSplitAngleV: array of Double;


       FLeaves: integer;
       FLeafScale, FLeafScaleX: Double;
      { FLeafQuality: Double; }        {LeafQuality determine # of Leaves
                                      *LeafQuality and Size  SQRT(LeafQuality)}
       FLeafAttractionUp: Double;
       FLeafOffset: Double; {Move leaf +- from stem}
       FLeafGrouping: byte;   {Range Box selection}
       FFLeafsPer: integer;  {Set}
       FLeafLocation: byte;  {Radio Group selection}
       FLeafImageFile  :String;
       FLeafShapeCount: byte;
       FLeafShape:array of  integer;
       FLeafShapeFile :array of String;
       {This is a  # and list of filenames
        to allow a variation of leaf textures}
       FLeafTextureCount: byte;
       FLeafTextureFile:array of String;
       FLeafColor: TColor;
       FLeafColorVariation: boolean;
       FLeafBackColor: TColor;
       FLeafBackColorVariation: boolean;

       FFruits : integer;  {~ Leaves..}
       FFruitScale, FFruitScaleX: Double;

       FFruitPer     : integer;  {#}
       FFruitLocation: byte;      {Radio Group selection}
       FFruitPattern      : byte;    {Range Box selection}
       FFruitImageFile   :String;
       FFruitShapeCount: byte;
       FFruitShapeFile   :array of String;
       FFruitShape      :array of  integer;    {Radio Group selection}
       FFruitTextureCount: byte;
       FFruitTextureFile :array of String;
       FFruitColor    : TColor;
       FFruitColorVariation : boolean;

       FPruneRatio: Double;
       FPruneWidth, FPruneWidthPeak: Double;
       FPrunePowerLow, FPrunePowerHigh: Double;

       function GetnDownAngle(Index: byte): Double;
       function GetnDownAngleV(Index: byte): Double;
       function GetnRotate(Index: byte): Double;
       function GetnRotateV(Index: byte): Double;
       function GetnLength(Index: byte): Double;
       function GetnLengthV(Index: byte): Double;
       function GetnTaper(Index: byte): Double;
       function GetnBranches(Index: byte): integer;
       function GetnSegSplits(Index: byte): Double;
       function GetnSplitAngle(Index: byte): Double;
       function GetnSplitAngleV(Index: byte): Double;
       function GetnCurveRes(Index: byte): integer;
       function GetnCurve(Index: byte): Double;
       function GetnCurveV(Index: byte): Double;
       function GetnCurveBack(Index: byte): Double;
       function GetnVertices(Index: byte): integer;
       function GetnLeaves(Index: byte): integer;
       procedure SetnDownAngle(Index: byte; Value: Double);
       procedure SetnDownAngleV(Index: byte; Value: Double);
       procedure SetnRotate(Index: byte; Value: Double);
       procedure SetnRotateV(Index: byte; Value: Double);
       procedure SetnLength(Index: byte; Value: Double);
       procedure SetnLengthV(Index: byte; Value: Double);
       procedure SetnTaper(Index: byte; Value: Double);
       procedure SetnBranches(Index: byte; Value: integer);
       procedure SetnSegSplits(Index: byte; Value: Double);
       procedure SetnSplitAngle(Index: byte; Value: Double);
       procedure SetnSplitAngleV(Index: byte; Value: Double);
       procedure SetnCurveRes(Index: byte; Value: integer);
       procedure SetnCurve(Index: byte; Value: Double);
       procedure SetnCurveV(Index: byte; Value: Double);
       procedure SetnCurveBack(Index: byte; Value: Double);
       procedure SetnVertices(Index: byte; Value: integer);
       procedure SetnLeaves(Index: byte; Value: integer);

       function GetLeafShape(Index: byte): integer;
       procedure SetLeafShape(Index: byte; Value: integer);
       function GetLeafShapeFile(Index: byte): String;
       procedure SetLeafShapeFile(Index: byte; Value: String);
       function GetLeafTextureFile(Index: byte): String;
       procedure SetLeafTextureFile(Index: byte; Value: String);

       function GetFruitShape(Index: byte): integer;
       procedure SetFruitShape(Index: byte; Value: integer);
       function GetFruitShapeFile(Index: byte): String;
       procedure SetFruitShapeFile(Index: byte; Value: String);
       function GetFruitTextureFile(Index: byte): String;
       procedure SetFruitTextureFile(Index: byte; Value: String);
     public
       { Model parameters }
       property Shape: byte read FShape write FShape;
       property BaseSize: Double read FBaseSize write FBaseSize;
       property BaseSplits: byte read FBaseSplits write FBaseSplits;
       property BoleSize: Double read FBoleSize write FBoleSize;
       property BoleSplits: byte read FBoleSplits write FBoleSplits;
       property DeathToll: byte read FDeathToll write FDeathToll;


       property Scale: Double read FScale write FScale;
       property ScaleV: Double read FScaleV write FScaleV;
       property ZScale: Double read FZScale write FZScale;
       property ZScaleV: Double read FZScaleV write FZScaleV;
       property Levelers: byte read FLeveled write FLeveled;
       property Ratio: Double read FRatio write FRatio;
       property RatioPower: Double read FRatioPower write FRatioPower;
       property Lobes: byte read FLobes write FLobes;
       property LobeDepth: Double read FLobeDepth write FLobeDepth;
       property LobeScale: Double read FLobeScale write FLobeScale;
       property LobeScaleV: Double read FLobeScaleV write FLobeScaleV;
       property Flare: Double read FFlare write FFlare;
       property FlareV: Double read FFlareV write FFlareV;
       property Leaves: integer read FLeaves write FLeaves;


       property PruneRatio: Double read FPruneRatio write FPruneRatio;
       property PruneWidth: Double read FPruneWidth write FPruneWidth;
       property PruneWidthPeak: Double read FPruneWidthPeak write FPruneWidthPeak;
       property PrunePowerLow: Double read FPrunePowerLow write FPrunePowerLow;
       property PrunePowerHigh: Double read FPrunePowerHigh write FPrunePowerHigh;
       property Scale0: Double read FScale0 write FScale0;
       property ScaleV0: Double read FScaleV0 write FScaleV0;
       property nDownAngle[Index: byte]: Double read GetnDownAngle write SetnDownAngle;
       property nDownAngleV[Index: byte]: Double read GetnDownAngleV write SetnDownAngleV;
       property nRotate[Index: byte]: Double read GetnRotate write SetnRotate;
       property nRotateV[Index: byte]: Double read GetnRotateV write SetnRotateV;
       property nLength[Index: byte]: Double read GetnLength write SetnLength;
       property nLengthV[Index: byte]: Double read GetnLengthV write SetnLengthV;
       property nTaper[Index: byte]: Double read GetnTaper write SetnTaper;
       property nBranches[Index: byte]: integer read GetnBranches write SetnBranches;
       property nSegSplits[Index: byte]: Double read GetnSegSplits write SetnSegSplits;
       property nSplitAngle[Index: byte]: Double read GetnSplitAngle write SetnSplitAngle;
       property nSplitAngleV[Index: byte]: Double read GetnSplitAngleV write SetnSplitAngleV;
       property nCurveRes[Index: byte]: integer read GetnCurveRes write SetnCurveRes;
       property nCurve[Index: byte]: Double read GetnCurve write SetnCurve;
       property nCurveBack[Index: byte]: Double read GetnCurveBack write SetnCurveBack;
       property nCurveV[Index: byte]: Double read GetnCurveV write SetnCurveV;
       property nVertices[Index: byte]: integer read GetnVertices write SetnVertices;
       {New added stuff}
       property TreeSeed : integer read FTreeSeed write FTreeSeed;
       property Health: byte read FHealth write FHealth;
       property Season: byte read FSeason write FSeason;
       property AttractionUp: Double read FAttractionUp write FAttractionUp;
       property TreeImageFile:String read FTreeImageFile write FTreeImageFile;
       property TreeShapeFile:String read FTreeShapeFile write FTreeShapeFile;
       property TreeBarkFile:String read FTreeBarkFile write FTreeBarkFile;
       property TreeSpriteFile:String read FTreeSpriteFile write FTreeSpriteFile;

       property TreeColorVariation : boolean read FTreeColorVariation write FTreeColorVariation;
       property TreeColor    : TColor read FTreeColor write FTreeColor;
       property Distance: Double read FDistance write FDistance;
       property WindStrength: byte read FWindStrength write FWindStrength;
       property WindGust: byte read FWindGust write FWindGust;
       property WindFrom : integer read FWindFrom write FWindFrom;
       property SunHeight: byte read FSunHeight write FSunHeight;
       property SunStrength: byte read FSunStrength write FSunStrength;
       property SunFrom  : integer read FSunFrom write FSunFrom;


                {Added so there can be Leaves on every stem}
       property LeafTextureCount: Byte read FLeafTextureCount write FLeafTextureCount;
       property LeafShapeCount: Byte read FLeafShapeCount write FLeafShapeCount;

      { property LeafShape: byte read FLeafShape write FLeafShape; }
       property LeafShape[Index: byte]: integer read GetLeafShape write SetLeafShape;

       property LeafScale: Double read FLeafScale write FLeafScale;
       property LeafScaleX: Double read FLeafScaleX write FLeafScaleX;
       property LeafColor: TColor read FLeafColor write FLeafColor;
       property LeafColorVariation: boolean read FLeafColorVariation write FLeafColorVariation;
       property LeafBackColor: TColor read FLeafBackColor write FLeafBackColor;
       property LeafBackColorVariation: boolean read FLeafBackColorVariation write FLeafBackColorVariation;
       property LeafAttractionUp: Double read FLeafAttractionUp write FLeafAttractionUp;

       property nLeaves[Index: byte]: integer read GetnLeaves write SetnLeaves;
       property LeafImageFile  :String read FLeafImageFile write FLeafImageFile;
       {property LeafShapeFile  :String read FLeafShapeFile write FLeafShapeFile;}
      property LeafShapeFile[Index: byte]:String
                             read GetLeafShapeFile write SetLeafShapeFile;

       {read FLeafTextureFile write FLeafTextureFile;}
       property LeafTextureFile[Index: byte]:String
                             read GetLeafTextureFile write SetLeafTextureFile;
       property LeafLocation: byte read FLeafLocation write FLeafLocation;
       property FLeafsPer: integer read FFLeafsPer write FFLeafsPer;
       property LeafGrouping: byte read FLeafGrouping write FLeafGrouping;
       property LeafOffset: Double read FLeafOffset write FLeafOffset;

       property FruitTextureCount: Byte read FFruitTextureCount write FFruitTextureCount;
       property FruitShapeCount: Byte read FFruitShapeCount write FFruitShapeCount;

       property FruitPattern: byte read FFruitPattern write FFruitPattern;
       property FruitScale: Double read FFruitScale write FFruitScale;
       property FruitScaleX: Double read FFruitScaleX write FFruitScaleX;
       property Fruits : integer read FFruits write FFruits;
       property FruitPer     : integer read FFruitPer write FFruitPer;
       property FruitLocation: byte read FFruitLocation write FFruitLocation;
       {property FruitShape: byte read FFruitShape write FFruitShape;}
       property FruitShape[Index: byte]: integer read GetFruitShape write SetFruitShape;

       property FruitImageFile   :String read FFruitImageFile write FFruitImageFile;
       {property FruitShapeFile   :String read FFruitShapeFile write FFruitShapeFile;}
      property FruitShapeFile[Index: byte]:String
                             read GetFruitShapeFile write SetFruitShapeFile;

       {property FruitTextureFile :String read FFruitTextureFile write FFruitTextureFile;}
       property FruitTextureFile[Index: byte]:String
                             read GetFruitTextureFile write SetFruitTextureFile;

       property FruitColorVariation : boolean read FFruitColorVariation write FFruitColorVariation;
       property FruitColor    : TColor read FFruitColor write FFruitColor;
     public
       constructor Create; overload;
       { Pre: True
         Post: A TModelParameters class has been created. None of the properties
               have been initialized. }
       constructor Create(Levels,
          InLeafTextureCount,InLeafShapeCount,
          InFruitTextureCount, InFruitShapeCount: byte); overload;
       { Pre: True
         Post: A TModelParameters class has been created. All the dynamic arrays
               have been created and set to the appropiate length
               (ALevels + 1) : 0..Levels }
       destructor Destroy; override;

       function Clones: TModelParameters;
       { Pre: True
         Ret: An exact copy of itself}
     end;

implementation

uses SysUtils;

constructor TModelParameters.Create;
{ Pre: True
  Post: A TModelParameters class has been created. None of the properties
        have been initialized. }
begin
  inherited Create;
end;

constructor TModelParameters.Create(Levels,
          InLeafTextureCount,InLeafShapeCount,
          InFruitTextureCount, InFruitShapeCount: byte);
{ Pre: True
  Post: A TModelParameters class has been created. All the dynamic arrays
        have been created and set to the appropiate length (ALevels + 1). }
begin
  inherited Create;

  FLeveled := Levels;
  { The array length of the Stem parameters is equal to ALevels.
    So, the total length of the stem parameter arrays is Levels - 1,
    0..Levels
    because the last level of recursion is reserved for the leaves. }
  SetLength(FnDownAngle, Levels);
  SetLength(FnDownAngleV, Levels);
  SetLength(FnRotate, Levels);
  SetLength(FnRotateV, Levels);
  SetLength(FnLength, Levels);
  SetLength(FnLengthV, Levels);
  SetLength(FnTaper, Levels);
  SetLength(FnBranches, Levels);
  SetLength(FnSegSplits, Levels);
  SetLength(FnSplitAngle, Levels);
  SetLength(FnSplitAngleV, Levels);
  SetLength(FnCurveRes, Levels);
  SetLength(FnCurve, Levels);
  SetLength(FnCurveBack, Levels);
  SetLength(FnCurveV, Levels);
  SetLength(FnVertices, Levels);
  SetLength(FnLeaves, Levels);{added}
  SetLength(FLeafTextureFile, InLeafTextureCount);
  SetLength(FLeafShapeFile, InLeafShapeCount);
  SetLength(FLeafShape, InLeafShapeCount);

  SetLength(FFruitTextureFile, InFruitTextureCount);
  SetLength(FFruitShapeFile, InFruitShapeCount);
  SetLength(FFruitShape, InFruitShapeCount);
{          InLeafTextureCount,InLeafShapeCount,
          InFruitTextureCount, InFruitShapeCount: byte);}
  {Possible Leaf..Fruit : Shape and Texture arrays}
end;


destructor TModelParameters.Destroy;
begin
  Finalize(FnDownAngle);
  Finalize(FnDownAngleV);
  Finalize(FnRotate);
  Finalize(FnRotateV);
  Finalize(FnLength);
  Finalize(FnTaper);
  Finalize(FnBranches);
  Finalize(FnSegSplits);
  Finalize(FnSplitAngle);
  Finalize(FnSplitAngleV);
  Finalize(FnCurveRes);
  Finalize(FnCurveBack);
  Finalize(FnCurve);
  Finalize(FnCurveV);
  Finalize(FnVertices);
  Finalize(FnLeaves);
  Finalize(FLeafTextureFile);
  Finalize(FLeafShapeFile);
  Finalize(FFruitTextureFile);
  Finalize(FFruitShapeFile);
  inherited Destroy;
end;

function TModelParameters.GetnBranches(Index: byte): integer;
{ Pre: 0 <= Index < Length(FnBranches)
  Ret: FnBranches[Index] }
begin
  Result := FnBranches[Index];
end;

function TModelParameters.GetnCurve(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnCurve)
  Ret: FnCurve[Index] }
begin
  Result := FnCurve[Index];
end;

function TModelParameters.GetnCurveBack(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnCurveBack)
  Ret: FnCurveBack[Index] }
begin
  Result := FnCurveBack[Index];
end;

function TModelParameters.GetnCurveRes(Index: byte): integer;
{ Pre: 0 <= Index < Length(FnCurveRes)
  Ret: FnCurveRes[Index] }
begin
  Result := FnCurveRes[Index];
end;

function TModelParameters.GetnCurveV(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnCurveV)
  Ret: FnCurveV[Index] }
begin
  Result := FnCurveV[Index];
end;

function TModelParameters.GetnDownAngle(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnDownAngle)
  Ret: FnDownAngle[Index] }
begin
  Result := FnDownAngle[Index];
end;

function TModelParameters.GetnDownAngleV(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnDownAngleV)
  Ret: FnDownAngleV[Index] }
begin
  Result := FnDownAngleV[Index];
end;

function TModelParameters.GetnLength(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnLength)
  Ret: FnLength[Index] }
begin
  Result := FnLength[Index];
end;

function TModelParameters.GetnLengthV(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnLengthV)
  Ret: FnLengthV[Index] }
begin
  Result := FnLengthV[Index];
end;

function TModelParameters.GetnRotate(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnRotate)
  Ret: FnRotate[Index] }
begin
  Result := FnRotate[Index];
end;

function TModelParameters.GetnRotateV(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnRotateV)
  Ret: FnRotateV[Index] }
begin
  Result := FnRotateV[Index];
end;

function TModelParameters.GetnSegSplits(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnSegSplits)
  Ret: FnSegSplits[Index] }
begin
  Result := FnSegSplits[Index];
end;

function TModelParameters.GetnSplitAngle(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnSplitAngle)
  Ret: FnSplitAngle[Index] }
begin
  Result := FnSplitAngle[Index];
end;

function TModelParameters.GetnSplitAngleV(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnSplitAngleV)
  Ret: FnSplitAngleV[Index] }
begin
  Result := FnSplitAngleV[Index];
end;

function TModelParameters.GetnTaper(Index: byte): Double;
{ Pre: 0 <= Index < Length(FnTaper)
  Ret: FnTaper[Index] }
begin
  Result := FnTaper[Index];
end;

function TModelParameters.GetnVertices(Index: byte): integer;
{ Pre: 0 <= Index < Length(FnVertices)
  Ret: FnVertices[Index] }
begin
  Result := FnVertices[Index];
end;

function TModelParameters.GetnLeaves(Index: byte): integer;
{ Pre: 0 <= Index < Length(FnLeaves)
  Ret: FnLeaves[Index] }
begin
  Result := FnLeaves[Index];
end;
procedure TModelParameters.SetnLeaves(Index: byte; Value: integer);
{ Pre: 0 <= Index < Length(FnLeaves)
  Post: FnLeavess[Index] = Value Leaves}
begin
  FnLeaves[Index] := Value;
end;

procedure TModelParameters.SetnBranches(Index: byte; Value: integer);
{ Pre: 0 <= Index < Length(FnBranches)
  Post: FnBranches[Index] = Value }
begin
  FnBranches[Index] := Value;
end;

procedure TModelParameters.SetnCurve(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnCurve)
  Post: FnCurve[Index] = Value }
begin
  FnCurve[Index] := Value;
end;

procedure TModelParameters.SetnCurveBack(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnCurveBack)
  Post: FnCurveBack[Index] = Value }
begin
  FnCurveBack[Index] := Value;
end;

procedure TModelParameters.SetnCurveRes(Index: byte; Value: integer);
{ Pre: 0 <= Index < Length(FnCurveRes)
  Post: FnCurveRes[Index] = Value }
begin
  FnCurveRes[Index] := Value;
end;

procedure TModelParameters.SetnCurveV(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnCurveV)
  Post: FnCurveV[Index] = Value }
begin
  FnCurveV[Index] := Value;
end;

procedure TModelParameters.SetnDownAngle(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnDownAngle)
  Post: FnDownAngle[Index] = Value }
begin
  FnDownAngle[Index] := Value;
end;

procedure TModelParameters.SetnDownAngleV(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnDownAngleV)
  Post: FnDownAngleV[Index] = Value }
begin
  FnDownAngleV[Index] := Value;
end;

procedure TModelParameters.SetnLength(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnLength)
  Post: FnLength[Index] = Value }
begin
  FnLength[Index] := Value;
end;

procedure TModelParameters.SetnLengthV(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnLengthV)
  Post: FnLengthV[Index] = Value }
begin
  FnLengthV[Index] := Value;
end;

procedure TModelParameters.SetnRotate(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnRotate)
  Post: FnRotate[Index] = Value }
begin
  FnRotate[Index] := Value;
end;

procedure TModelParameters.SetnRotateV(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnRotateV)
  Post: FnRotateV[Index] = Value }
begin
  FnRotateV[Index] := Value;
end;

procedure TModelParameters.SetnSegSplits(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnSegSplits)
  Post: FnSegSplits[Index] = Value }
begin
  FnSegSplits[Index] := Value;
end;

procedure TModelParameters.SetnSplitAngle(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnSplitAngle)
  Post: FnSplitAngle[Index] = Value }
begin
  FnSplitAngle[Index] := Value;
end;

procedure TModelParameters.SetnSplitAngleV(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnSplitAngleV)
  Post: FnSplitAngleV[Index] = Value }
begin
  FnSplitAngleV[Index] := Value;
end;

procedure TModelParameters.SetnTaper(Index: byte; Value: Double);
{ Pre: 0 <= Index < Length(FnTaper)
  Post: FnTaper[Index] = Value }
begin
  FnTaper[Index] := Value;
end;

procedure TModelParameters.SetnVertices(Index: byte; Value: integer);
{ Pre: 0 <= Index < Length(FnVertices)
  Post: FnVertices[Index] = Value }
begin
  FnVertices[Index] := Value;
end;


function TModelParameters.GetLeafShape(Index: byte): integer;
{ Pre: 0 <= Index < Length(FLeafShape)
  Ret: FLeafShape[Index] }
begin
  Result := FLeafShape[Index];
end;
procedure TModelParameters.SetLeafShape(Index: byte; Value: integer);
{ Pre: 0 <= Index < Length(FLeafShape)
  Post: FLeafShape[Index] = Value }
begin
  FLeafShape[Index] := Value;
end;
function TModelParameters.GetLeafShapeFile(Index: byte): String;
{ Pre: LeafShapeFile unk
  Ret: FLeafShapeFile[Index] }
begin
  Result := FLeafShapeFile[Index];
end;
procedure TModelParameters.SetLeafShapeFile(Index: byte; Value: String);
{ Pre: LeafShapeFile unk
  Post: FLeafTShapeFile[Index] = Value }
begin
  FLeafShapeFile[Index] := Value;
end;
function TModelParameters.GetLeafTextureFile(Index: byte): String;
{ Pre: LeafTextureFile unk
  Ret: FLeafTextureFile[Index] }
begin
  Result := FLeafTextureFile[Index];
end;
procedure TModelParameters.SetLeafTextureFile(Index: byte; Value: String);
{ Pre: LeafTextureFile unk
  Post: FLeafTextureFile[Index] = Value }
begin
  FLeafTextureFile[Index] := Value;
end;

function TModelParameters.GetFruitShape(Index: byte): integer;
{ Pre: 0 <= Index < Length(FFruitShape)
  Ret: FFruitShape[Index] }
begin
  Result := FFruitShape[Index];
end;
procedure TModelParameters.SetFruitShape(Index: byte; Value: integer);
{ Pre: 0 <= Index < Length(FFruitShape)
  Post: FFruitShape[Index] = Value }
begin
  FFruitShape[Index] := Value;
end;
function TModelParameters.GetFruitShapeFile(Index: byte): String;
{ Pre: FruitShapeFile unk
  Ret: FFruitShapeFile[Index] }
begin
  Result := FFruitShapeFile[Index];
end;
procedure TModelParameters.SetFruitShapeFile(Index: byte; Value: String);
{ Pre: FruitShapeFile unk
  Post: FFruitShapeFile[Index] = Value }
begin
  FFruitShapeFile[Index] := Value;
end;
function TModelParameters.GetFruitTextureFile(Index: byte): String;
{ Pre: FruitTextureFile unk
  Ret: FFruitTextureFile[Index] }
begin
  Result := FFruitTextureFile[Index];
end;
procedure TModelParameters.SetFruitTextureFile(Index: byte; Value: String);
{ Pre: FruitTextureFile unk
  Post: FFruitTextureFile[Index] = Value }
begin
  FFruitTextureFile[Index] := Value;
end;

function TModelParameters.Clones: TModelParameters;
{ Pre: True
  Ret: An exact copy of itself. }
var i: integer;
begin
  Result := TModelParameters.Create(Self.Levelers,
          Self.LeafTextureCount,Self.LeafShapeCount,
          Self.FruitTextureCount, Self.FruitShapeCount);
            Result.TreeSeed := FTreeSeed;

  Result.Shape := FShape;
  Result.Levelers := FLeveled; {This is the trick: Done in Create}
  Result.Season := FSeason;
  Result.AttractionUp := FAttractionUp;
  Result.TreeSeed := FTreeSeed;
  Result.Health := FHealth;
  Result.DeathToll := FDeathToll;

  Result.Distance := FDistance;
  Result.WindStrength := FWindStrength;
  Result.WindGust := FWindGust;
  Result.WindFrom := FWindFrom;
  Result.SunHeight := FSunHeight;
  Result.SunStrength := FSunStrength;
  Result.SunFrom := FSunFrom;

  Result.TreeImageFile := FTreeImageFile;
  Result.TreeShapeFile := FTreeShapeFile;
  Result.TreeSpriteFile := FTreeSpriteFile;

  Result.TreeBarkFile := FTreeBarkFile;
  Result.TreeColorVariation := FTreeColorVariation;
  Result.TreeColor := FTreeColor;
  Result.Scale := FScale;
  Result.ScaleV := FScaleV;
       {FnLength}
       {FnLengthV}



  Result.Ratio := FRatio;
  Result.FRatioPower := FRatioPower;
  Result.Scale0 := FScale0;
  Result.ScaleV0 := FScaleV0;
  Result.Lobes := FLobes;
  Result.LobeDepth := FLobeDepth;
  Result.LobeScale := FLobeScale;
  Result.LobeScaleV := FLobeScaleV;

  Result.BaseSize := FBaseSize;
  Result.BaseSplits := FBaseSplits;
  Result.ZScale := FZScale;
  Result.ZScaleV := FZScaleV;

  Result.BoleSize := FBoleSize;
  Result.BoleSplits := FBoleSplits;
  Result.Flare := FFlare;
  Result.FlareV := FFlareV;

  for i := 0 to Levelers - 1 do
  begin
    Result.nDownAngle[i] := FnDownAngle[i];
    Result.nDownAngleV[i] := FnDownAngleV[i];
    Result.nRotate[i] := FnRotate[i];
    Result.nRotateV[i] := FnRotateV[i];
    Result.nLength[i] := FnLength[i];
    Result.nLengthV[i] := FnLengthV[i];
    Result.nTaper[i] := FnTaper[i];
    Result.nBranches[i] := FnBranches[i];
    Result.nSegSplits[i] := FnSegSplits[i];
    Result.nSplitAngle[i] := FnSplitAngle[i];
    Result.nSplitAngleV[i] := FnSplitAngleV[i];
    Result.nCurveRes[i] := FnCurveRes[i];
    Result.nCurveBack[i] := FnCurveBack[i];
    Result.nCurve[i] := FnCurve[i];
    Result.nCurveV[i] := FnCurveV[i];
    Result.nVertices[i] := FnVertices[i];
    Result.nLeaves[i] := FnLeaves[i];
  end;


  Result.Leaves := FLeaves;
  Result.LeafScale := FLeafScale;
  Result.LeafScaleX := FLeafScaleX;
  Result.LeafAttractionUp := FLeafAttractionUp;
  Result.LeafGrouping := FLeafGrouping;
  Result.LeafOffset := FLeafOffset;
  Result.LeafLocation := FLeafLocation;
  Result.FLeafsPer := FFLeafsPer;

  Result.LeafImageFile := FLeafImageFile;
  Result.LeafShapeCount := FLeafShapeCount;
  for i := 0 to LeafShapeCount - 1 do
  begin
    Result.LeafShapeFile[i] := FLeafShapeFile[i];
    Result.LeafShape[i] := FLeafShape[i];
  end;

  Result.LeafTextureCount := FLeafTextureCount;
  for i := 0 to FLeafTextureCount - 1 do
    Result.LeafTextureFile[i] := FLeafTextureFile[i];

  Result.LeafColor := FLeafColor;
  Result.LeafColorVariation := FLeafColorVariation;
  Result.LeafBackColor := FLeafBackColor;
  Result.LeafBackColorVariation := FLeafBackColorVariation;

  Result.Fruits := FFruits;
  Result.FruitPattern := FFruitPattern;
  Result.FruitScale := FFruitScale;
  Result.FruitScaleX := FFruitScaleX;
  Result.FruitPer := FFruitPer;

  Result.FruitShapeCount := FFruitShapeCount;
  for i := 0 to FruitShapeCount - 1 do
  begin
  Result.FruitShape[i] := FFruitShape[i];
  Result.FruitShapeFile[i] := FFruitShapeFile[i];
  end;
  Result.FruitLocation := FFruitLocation;
  Result.FruitImageFile := FFruitImageFile;


  Result.FruitTextureCount := FFruitTextureCount;
  for i := 0 to FruitTextureCount - 1 do
    Result.FruitTextureFile[i] := FFruitTextureFile[i];
  Result.FruitColorVariation := FFruitColorVariation;
  Result.FruitColor := FFruitColor;

  Result.PruneRatio := FPruneRatio;
  Result.PruneWidth := FPruneWidth;
  Result.PruneWidthPeak := FPruneWidthPeak;
  Result.PrunePowerLow := FPrunePowerLow;
  Result.PrunePowerHigh := FPrunePowerHigh;
end;

end.
